/*
 * SpecificFragmentController_impl.hpp
 *
 *  Created on: Aug 27, 2012
 *      Author: heber
 */

#ifndef SPECIFICFRAGMENTCONTROLLER_IMPL_HPP_
#define SPECIFICFRAGMENTCONTROLLER_IMPL_HPP_


// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "SpecificFragmentController.hpp"

#include "CodePatterns/Assert.hpp"
#include "CodePatterns/toString.hpp"


template <typename T>
void SpecificFragmentController::getResults(std::map<JobId_t, T> &IdData)
{
  // receive
  receiveResults(host, port);
  RunService("Requesting results");
  std::vector<FragmentResult::ptr> fragmentresults = getReceivedResults();
  // convert
  std::vector<T> fragmentData;
  ConvertFragmentResultTo<T>(fragmentresults, fragmentData);
  // put into map
  IdData.clear();
  std::vector<FragmentResult::ptr>::const_iterator resultiter = fragmentresults.begin();
  typename std::vector<T>::const_iterator dataiter = fragmentData.begin();
  for(;resultiter != fragmentresults.end(); ++resultiter, ++dataiter) {
#ifndef NDEBUG
    std::pair<typename std::map<JobId_t, T>::iterator, bool> inserter =
#endif
    IdData.insert( std::make_pair( (*resultiter)->getId(), *dataiter) );
    ASSERT( inserter.second,
        "SpecificFragmentController::getResults() - result id "
        +toString(inserter.first->first)+" already present.");
  }
  ASSERT( dataiter == fragmentData.end(),
      "SpecificFragmentController::getResults() - fragmentresults and fragmentdata differ in size.");
}

template <typename T>
void SpecificFragmentController::ConvertFragmentResultTo(
    const std::vector<FragmentResult::ptr> &results,
    std::vector<T> &fragmentData)
{
  // extract results
  fragmentData.clear();
  fragmentData.reserve(results.size());

  LOG(2, "DEBUG: Parsing now through " << results.size() << " results.");
  for (std::vector<FragmentResult::ptr>::const_iterator iter = results.begin();
      iter != results.end(); ++iter) {
    //LOG(1, "RESULT: job #"+toString((*iter)->getId())+": "+toString((*iter)->result));
    T extractedData;
    std::stringstream inputstream((*iter)->result);
    LOG(2, "DEBUG: First 50 characters FragmentResult's string: "+(*iter)->result.substr(0, 50));
    boost::archive::text_iarchive ia(inputstream);
    ia >> extractedData;
    LOG(1, "INFO: extracted data is " << extractedData << ".");
    fragmentData.push_back(extractedData);
  }

  ASSERT( results.size() == fragmentData.size(),
      "ConvertFragmentResultTo() - the number of extracted data differs from the number of results.");
}


#endif /* SPECIFICFRAGMENTCONTROLLER_IMPL_HPP_ */
