/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * VMGFragmentController.cpp
 *
 *  Created on: Aug 27, 2012
 *      Author: heber
 */


// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// boost asio needs specific operator new
#include <boost/asio.hpp>

#include "CodePatterns/MemDebug.hpp"

#include "VMGFragmentController.hpp"

#include "Atom/atom.hpp"
#include "Element/element.hpp"
#include "Jobs/VMGJob.hpp"
#include "molecule.hpp"
#include "World.hpp"

bool VMGFragmentController::createLongRangeJobs(
    const std::map<JobId_t, MPQCData> &fragmentData,
    const SamplingGrid &full_sampled_grid,
    const Fragment &full_fragment)
{
  std::vector<FragmentJob::ptr> jobs;
  // add one job for each fragment as the short-range correction which we need
  // to subtract from the obtained full potential to get the long-range part only
  for (std::map<JobId_t, MPQCData>::const_iterator iter = fragmentData.begin();
      iter != fragmentData.end(); ++iter) {
    const JobId_t next_id = getAvailableId();
    LOG(1, "INFO: Creating VMGJob with " << iter->second.sampled_grid.sampled_grid.size()
        << " gridpoints and " << iter->second.charges.size() << " particle charges.");
    FragmentJob::ptr testJob(
        new VMGJob(next_id, iter->second.sampled_grid, iter->second.positions, iter->second.charges) );
    jobs.push_back(testJob);
  }

  {
    const World::AtomComposite &atoms = World::getInstance().getAllAtoms();
    std::vector< std::vector<double> > positions;
    positions.reserve(atoms.size());
    std::vector<double> charges;
    charges.reserve(atoms.size());
    std::vector<double> position(3, 0.);
    for (World::AtomComposite::const_iterator iter = atoms.begin();
        iter != atoms.end(); ++iter) {
      const Vector &pos = (*iter)->getPosition();
      for (size_t i=0;i<3;++i) position[i] = pos[i];
      positions.push_back(position);
      charges.push_back((double)((*iter)->getElement().getAtomicNumber()));
    }
    const JobId_t next_id = getAvailableId();
    LOG(1, "INFO: Creating full VMGJob with " << full_sampled_grid.sampled_grid.size()
        << " gridpoints and " << charges.size() << " particle charges.");
    FragmentJob::ptr testJob(
        new VMGJob(next_id, full_sampled_grid, positions, charges) );
    jobs.push_back(testJob);
  }

  // then send jobs to controller
  addJobs(jobs);
  sendJobs(host, port);
  RunService("Adding VMGJobs");

  return true;
}
